package demo;

import java.util.ArrayList;

import javafx.animation.AnimationTimer;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;

public class DemoController {

	@FXML
	private Pane pane;

	@FXML
	private Circle ball1;

	@FXML
	private Circle ball2;

	private Movement clock;

	private ArrayList<Ball> juggling;
	private ArrayList<Ball> waiting;

	private long FRAMES_PER_SEC = 50L;
	private long INTERVAL = 1000000000L / FRAMES_PER_SEC;

	private class Movement extends AnimationTimer {

		private long last = 0;
		private ArrayList<Ball> bs;

		public void setBalls(ArrayList<Ball> bs) {
			this.bs = bs;
		}

		@Override
		public void handle(long now) {

			// If enough time has passed:
			if (now - last > INTERVAL) {

				// Move and draw all the balls
				for (Ball b : bs) {
					b.move();
					b.draw();
				}

				// reset the interval start time
				last = now;
			}
		}
	}

	@FXML
	public void initialize() {
		ball1.setOnMouseDragged(event -> drag(event));
		ball2.setOnMouseDragged(event -> drag(event));
		ball1.setOnMousePressed(event -> pressed(event));
		ball2.setOnMousePressed(event -> pressed(event));
		ball1.setOnMouseReleased(event -> released(event));
		ball2.setOnMouseReleased(event -> released(event));

		juggling = new ArrayList<Ball>();
		waiting = new ArrayList<Ball>();
		juggling.add(new Ball(35, 32, 0, 1, 25, ball1, pane, true));
		juggling.add(new Ball(45, 30, 1, 0, 15, ball2, pane, false));


		for (int i = 0; i < 10; i++) {
			Circle c = new Circle();
			c.setFill(Color.GREEN);
			c.setStroke(Color.BLACK);
			c.setOnMouseDragged(event -> drag(event));
			c.setOnMousePressed(event -> pressed(event));
			c.setOnMouseReleased(event -> released(event));
			juggling.add(new Ball(35 + i * 4, 32 + i * 4, 1, 1, 25, c, pane, true));
			pane.getChildren().add(c);
		}

		clock = new Movement();
		clock.setBalls(juggling);
		clock.start();
	}

	// What to do when the mouse is Pressed
	public void pressed(MouseEvent event) {
		Node n = (Node) event.getSource();
		Ball b = findBall(n, juggling);
		if (b != null) {
			juggling.remove(b);
			waiting.add(b);
			b.setColor(Color.DARKRED);
		}
	}

	// What to do when the mouse is Released
	public void released(MouseEvent event) {
		Node n = (Node) event.getSource();
		Ball b = findBall(n, waiting);
		if (b != null) {
			waiting.remove(b);
			juggling.add(b);
			b.setColor(Color.DODGERBLUE);
		}
	}

	// What to do when the mouse is Dragged
	public void drag(MouseEvent event) {
		Node n = (Node) event.getSource();
		Ball b = findBall(n, waiting);
		if (b != null) {
			b.setX(b.getX() + event.getX());
			b.setY(b.getY() + event.getY());
			b.draw();
		}
	}

	// Look for the ball associated with a Node in a given ArrayList
	public Ball findBall(Node n, ArrayList<Ball> list) {
		for (Ball b : list) {
			if (b.getCircle().equals(n)) {
				return b;
			}
		}
		return null;
	}
}
